@@ -19,10 +19,6 @@
 */
 
 #include <config.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
 
 #include <glib/gi18n.h>
 
@@ -33,12 +29,13 @@
 #include <libanjuta/interfaces/ianjuta-file.h>
 #include <libanjuta/interfaces/ianjuta-file-loader.h>
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
+#include <libanjuta/interfaces/ianjuta-file-manager.h>
+#include <libanjuta/interfaces/ianjuta-project-manager.h>
 #include <libanjuta/interfaces/ianjuta-wizard.h>
-#include <libegg/menu/egg-submenu-action.h>
-#include <libegg/menu/egg-recent-action.h>
 
 #include "plugin.h"
 #include "dnd.h"
+#include "anjuta-recent-chooser-menu.h"
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-loader-plugin.ui"
 
@@ -66,47 +63,61 @@ sort_wizards(gconstpointer wizard1, gconstpointer wizard2)
 		return 0;
 }
 
+/* The add argument is here to remember that recent menu items should be removed
+ * when the destination does not exist anymore */
 static void
-set_recent_file (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
-				 const gchar *mime)
+update_recent_file (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
+				 const gchar *mime, gboolean add)
 {
-	EggRecentItem *recent_item;
-	DEBUG_PRINT ("Adding recent item of mimi-type: %s", mime);
-	recent_item = egg_recent_item_new ();
-	egg_recent_item_set_uri (recent_item, uri);
-	egg_recent_item_set_mime_type (recent_item, mime);
-	if (strcmp (mime, "application/x-anjuta") == 0)
+	
+	if (add)
 	{
-		egg_recent_item_add_group (recent_item, "anjuta-projects");
-		egg_recent_model_add_full (plugin->recent_files_model_top,
-								   recent_item);
+		GtkRecentData *recent_data;
+	
+		recent_data = g_slice_new (GtkRecentData);
+
+		recent_data->display_name = NULL;
+		recent_data->description = NULL;
+		recent_data->mime_type = (gchar *)mime;
+		recent_data->app_name = (gchar *) g_get_application_name ();
+		recent_data->app_exec = g_strjoin (" ", g_get_prgname (), "%u", NULL);
+		recent_data->groups = NULL;
+		recent_data->is_private = FALSE;
+	
+		if (!gtk_recent_manager_add_full (plugin->recent_manager, uri, recent_data))
+		{
+      		g_warning ("Unable to add '%s' to the list of recently used documents", uri);
+		}
+
+		g_free (recent_data->app_exec);
+		g_slice_free (GtkRecentData, recent_data);
 	}
 	else
 	{
-		egg_recent_item_add_group (recent_item, "anjuta-files");
-		egg_recent_model_add_full (plugin->recent_files_model_bottom,
-								   recent_item);
+		gtk_recent_manager_remove_item (plugin->recent_manager, uri, NULL);
 	}
+
 }
 
 static void
 launch_application_failure (AnjutaFileLoaderPlugin *plugin,
 							const gchar *uri,
-							GnomeVFSResult result)
+							const gchar *errmsg)
 {
-	const gchar *errmsg;
 	GtkWidget *parent;
+	gchar *basename;
 	
-	errmsg = gnome_vfs_result_to_string (result);
 	parent =
 		gtk_widget_get_toplevel (GTK_WIDGET(ANJUTA_PLUGIN (plugin)->shell));
+	basename = g_path_get_basename (uri);
 	anjuta_util_dialog_error (GTK_WINDOW (parent),
-							  "Can not open \"%s\".\n\n%s",
-							  g_basename (uri), errmsg);
+							  _("Can not open \"%s\".\n\n%s"),
+							  basename, errmsg);
+	g_free (basename);
 }
 
 static GList *
-get_available_plugins_for_mime (AnjutaFileLoaderPlugin* plugin,
+get_available_plugins_for_mime (AnjutaPlugin* plugin,
 							   const gchar *mime_type)
 {
 	AnjutaPluginManager *plugin_manager;
@@ -114,8 +125,10 @@ get_available_plugins_for_mime (AnjutaFileLoaderPlugin* plugin,
 	
 	g_return_val_if_fail (mime_type != NULL, NULL);
 	
-	plugin_manager = anjuta_shell_get_plugin_manager (ANJUTA_PLUGIN(plugin)->shell,
+	plugin_manager = anjuta_shell_get_plugin_manager (plugin->shell,
 													  NULL);
+	
+	/* Check an exact match */
 	plugin_descs = anjuta_plugin_manager_query (plugin_manager,
 												"Anjuta Plugin",
 												"Interfaces", "IAnjutaFile",
@@ -123,17 +136,55 @@ get_available_plugins_for_mime (AnjutaFileLoaderPlugin* plugin,
 												"SupportedMimeTypes",
 												mime_type,
 												NULL);
-	if (!plugin_descs);
+	
+	/* Check for plugins supporting one supertype */
+	if (plugin_descs == NULL)
 	{
-		gchar* supertype = gnome_vfs_get_supertype_from_mime_type (mime_type);
-		plugin_descs = anjuta_plugin_manager_query (plugin_manager,
+		GList *node;
+		GList *loader_descs = NULL;	
+		
+		loader_descs = anjuta_plugin_manager_query (plugin_manager,
 												"Anjuta Plugin",
 												"Interfaces", "IAnjutaFile",
-												"File Loader",
-												"SupportedMimeTypes",
-												supertype,
 												NULL);
-		g_free (supertype);
+		for (node = g_list_first (loader_descs); node != NULL; node = g_list_next (node))
+		{
+			gchar *value;
+			
+			if (anjuta_plugin_description_get_string ((AnjutaPluginDescription *)node->data,
+													  "File Loader", "SupportedMimeTypes", &value))
+			{
+				gchar **split_value;
+				
+				split_value = g_strsplit (value, ",", -1);
+				g_free (value);
+				if (split_value)
+				{
+					gchar **mime;
+					
+					for (mime = split_value; *mime != NULL; mime++)
+					{
+						/* The following line is working on unix only where
+						 * content and mime type are the same. Normally the
+						 * mime type has to be converted to a content type.
+						 * But it is a recent (glib 2.18) function, I think we can
+						 * wait a bit to fix this */
+						if (g_content_type_is_a (mime_type, *mime))
+						{
+							gchar *loc;
+							anjuta_plugin_description_get_string ((AnjutaPluginDescription *)node->data,
+													  "Anjuta Plugin", "Location", &loc);
+							
+							plugin_descs = g_list_prepend (plugin_descs, node->data);
+							break;
+						}
+					}
+				}
+				g_strfreev (split_value);
+			}
+		}
+		g_list_free (loader_descs);
+		plugin_descs = g_list_reverse (plugin_descs);
 	}
 	
 	return plugin_descs;
@@ -146,24 +197,28 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 	GList *plugin_descs, *snode;
 	GList *mime_apps, *node;
 	GtkWidget *menu, *menuitem;
-	GnomeVFSMimeApplication *mime_app;
+	GAppInfo *mime_app;
 	
 	GtkWidget *dialog, *parent, *hbox, *label;
 	GtkWidget *options;
 	gchar *message;
+	gchar *basename;
 	AnjutaPluginManager *plugin_manager;
 	
 	plugin_manager = anjuta_shell_get_plugin_manager (ANJUTA_PLUGIN (plugin)->shell,
 													  NULL);
+	basename = g_path_get_basename (uri);
 	message = g_strdup_printf (_("<b>Cannot open \"%s\"</b>.\n\n"
 								 "There is no plugin, default action, or application "
 								 "configured to handle this file type.\n"
 								 "\n"
-								 "Mime type: %s.\n"
+								 "Mime type: %s\n"
 								 "\n"
 								 "You may choose to try opening it with the following "
 								 "plugins or applications."),
-							   g_basename(uri), mime_type);
+							   basename, mime_type);
+	g_free (basename);
+
 	parent =
 		gtk_widget_get_toplevel (GTK_WIDGET(ANJUTA_PLUGIN (plugin)->shell));
 	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (parent),
@@ -187,7 +242,7 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 	gtk_menu_append (menu, menuitem);
 	
 	/* Open with plugins menu items */
-	plugin_descs = get_available_plugins_for_mime (plugin, mime_type);
+	plugin_descs = get_available_plugins_for_mime (ANJUTA_PLUGIN (plugin), mime_type);
 	snode = plugin_descs;
 	while (snode)
 	{
@@ -218,7 +273,7 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 	}
 	
 	/* Open with application menu items */
-	mime_apps = gnome_vfs_mime_get_all_applications (mime_type);
+	mime_apps = g_app_info_get_all_for_type (mime_type);
 	if (mime_apps)
 	{
 		/* Separator */
@@ -228,9 +283,13 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 	node = mime_apps;
 	while (node)
 	{
-		mime_app = (GnomeVFSMimeApplication *)(node->data);
-		menuitem = gtk_menu_item_new_with_label (mime_app->name);
-		gtk_menu_append (menu, menuitem);
+		mime_app = (GAppInfo *)(node->data);
+		if (g_app_info_should_show (mime_app))
+		{
+			menuitem = gtk_menu_item_new_with_label (
+					g_app_info_get_name (mime_app));
+			gtk_menu_append (menu, menuitem);
+		}
 		node = g_list_next (node);
 	}
 	
@@ -250,7 +309,9 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 												 NULL);
 			if (docman)
 			{
-				ianjuta_file_open (IANJUTA_FILE (docman), uri, NULL);
+				GFile* file = g_file_new_for_uri (uri);
+				ianjuta_file_open (IANJUTA_FILE (docman), file, NULL);
+				g_object_unref (file);
 			}
 			else
 			{
@@ -276,8 +337,10 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 															location);
 				if (loaded_plugin)
 				{
-					ianjuta_file_open (IANJUTA_FILE (loaded_plugin), uri, NULL);
-					set_recent_file (plugin, uri, mime_type);
+					GFile* file = g_file_new_for_uri (uri);
+					ianjuta_file_open (IANJUTA_FILE (loaded_plugin), file, NULL);
+					update_recent_file (plugin, uri, mime_type, TRUE);
+					g_object_unref (file);
 				}
 				else
 				{
@@ -291,105 +354,76 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 		else
 		{
 			GList *uris = NULL;
-			GnomeVFSResult res;
+			GError *error = NULL;
 			
 			option -= (g_list_length (plugin_descs) + 2);
 			mime_app = g_list_nth_data (mime_apps, option);
 			uris = g_list_prepend (uris, (gpointer)uri);
-			res = gnome_vfs_mime_application_launch (mime_app, uris);
-			if (res != GNOME_VFS_OK)
-				launch_application_failure (plugin, uri, res);
-			else
-				set_recent_file (plugin, uri, mime_type);
+			g_app_info_launch_uris(mime_app, uris, NULL, &error);
+			if (error)
+			{
+				launch_application_failure (plugin, uri, error->message);
+				g_error_free (error);
+			}
+			update_recent_file (plugin, uri, mime_type, error == NULL);
 			g_list_free (uris);
 		}
 	}
-	gnome_vfs_mime_application_list_free (mime_apps);
+	g_list_foreach (mime_apps, (GFunc) g_object_unref, NULL);
+	g_list_free (mime_apps);
 	if (plugin_descs)
 		g_list_free (plugin_descs);
 	gtk_widget_destroy (dialog);
 }
 
-static gboolean
-launch_in_default_application (AnjutaFileLoaderPlugin *plugin,
-							   const gchar *mime_type, const gchar *uri)
-{
-	GnomeVFSMimeAction *action;
-	GList *uris = NULL;
-	gboolean ret;
-	
-	uris = g_list_prepend (uris, (gpointer)uri);
-	
-	ret = TRUE;
-	action = gnome_vfs_mime_get_default_action (mime_type);
-	if (!action || gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK)
-	{
-		GnomeVFSMimeApplication *app;
-		GnomeVFSResult res;
-		app = gnome_vfs_mime_get_default_application (mime_type);
-		if (!app ||
-			(res =
-				gnome_vfs_mime_application_launch (app, uris)) != GNOME_VFS_OK)
-		{
-			open_with_dialog (plugin, uri, mime_type);
-		}
-		if (app)
-			gnome_vfs_mime_application_free (app);
-	}
-	if (action)
-		gnome_vfs_mime_action_free (action);
-	g_list_free (uris);
-	return ret;
-}
-
 static void
 open_file (AnjutaFileLoaderPlugin *plugin, const gchar *uri)
 {
-	GnomeVFSURI *vfs_uri;	
 	gchar *dirname;
+	gchar *path;
+	GFile* file;
 	
-	vfs_uri = gnome_vfs_uri_new (uri);
-	dirname = gnome_vfs_uri_extract_dirname (vfs_uri);
-	gnome_vfs_uri_unref (vfs_uri);
+	file = g_file_new_for_uri (uri);
+	path = g_file_get_path (file);
+	dirname = g_path_get_dirname (uri);
 	chdir (dirname);
+	g_free (path);
 	g_free (dirname);
 	
 	ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin),
-							  uri, FALSE, NULL);
+							  file, FALSE, NULL);
+	g_object_unref (file);
 }
 
 typedef struct
 {
 	AnjutaFileLoaderPlugin *plugin;
 	const gchar *uri;
-} RecentIdelOpenData;
+} RecentIdleOpenData;
 
 static gboolean
 on_open_recent_file_idle (gpointer data)
 {
-	RecentIdelOpenData *rdata;
+	RecentIdleOpenData *rdata;
 	
-	rdata = (RecentIdelOpenData*)data;
+	rdata = (RecentIdleOpenData*)data;
 	open_file (rdata->plugin, rdata->uri);
 	g_free (rdata);
 	return FALSE;
 }
 
 static gboolean
-on_open_recent_file (EggRecentAction *action, AnjutaFileLoaderPlugin *plugin)
+on_open_recent_file (GtkRecentChooser *chooser, AnjutaFileLoaderPlugin *plugin)
 {
 	const gchar *uri;
-	GnomeVFSURI *vfs_uri;
 	gboolean ret = TRUE;
-	RecentIdelOpenData *rdata;
+	RecentIdleOpenData *rdata;
 	
-	uri = egg_recent_action_get_selected_uri (action);
-	vfs_uri = gnome_vfs_uri_new (uri);
-	rdata = g_new0 (RecentIdelOpenData, 1);
+	uri = gtk_recent_chooser_get_current_uri (chooser);
+	rdata = g_new0 (RecentIdleOpenData, 1);
 	rdata->plugin = plugin;
 	rdata->uri = uri;
 	g_idle_add (on_open_recent_file_idle, rdata);
-	gnome_vfs_uri_unref (vfs_uri);
 
 	return ret;
 }
@@ -526,6 +560,9 @@ create_file_open_dialog_gui(GtkWindow* parent, AnjutaFileLoaderPlugin* plugin)
 									NULL);
 	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER(dialog), TRUE);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
+	gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
+	gtk_window_set_destroy_with_parent (GTK_WINDOW(dialog), TRUE);
+	
 	setup_file_filters (GTK_FILE_CHOOSER (dialog));
 	
 	g_signal_connect(G_OBJECT(dialog), "response", 
@@ -537,6 +574,39 @@ create_file_open_dialog_gui(GtkWindow* parent, AnjutaFileLoaderPlugin* plugin)
 	return dialog;
 }
 
+static void
+on_new_clicked (GtkToolButton *button, AnjutaFileLoaderPlugin *plugin)
+{
+	AnjutaShell* shell = ANJUTA_PLUGIN (plugin)->shell;
+	IAnjutaDocumentManager *docman = anjuta_shell_get_interface (shell, 
+																 IAnjutaDocumentManager,
+																 NULL);
+	if (docman)
+		ianjuta_document_manager_add_buffer (docman, NULL, NULL, NULL);
+}
+
+static void
+on_open_clicked (GtkToolButton *button, AnjutaFileLoaderPlugin *plugin)
+{
+	GtkWidget *dlg;
+	
+	dlg =
+		create_file_open_dialog_gui (GTK_WINDOW (ANJUTA_PLUGIN(plugin)->shell),
+									 plugin);
+	gtk_widget_show (dlg);
+}
+
+static void
+on_new_activate (GtkAction *action, AnjutaFileLoaderPlugin *plugin)
+{
+	AnjutaShell* shell = ANJUTA_PLUGIN (plugin)->shell;
+	IAnjutaDocumentManager *docman = anjuta_shell_get_interface (shell, 
+																 IAnjutaDocumentManager,
+																 NULL);
+	if (docman)
+		ianjuta_document_manager_add_buffer (docman, NULL, NULL, NULL);
+}
+
 static void
 on_open_activate (GtkAction *action, AnjutaFileLoaderPlugin *plugin)
 {
@@ -665,45 +735,26 @@ on_create_submenu (gpointer user_data)
 	return submenu;
 }
 
-/*
 static void
-on_recent_files_tooltip (GtkTooltips *tooltips, GtkWidget *menu_item,
-						 EggRecentItem *item, gpointer user_data)
-{
-	char *uri, *tip;
-
-	uri = egg_recent_item_get_uri_for_display (item);
-	tip = g_strdup_printf ("Open '%s'", uri);
-
-	gtk_tooltips_set_tip (tooltips, menu_item, tip, NULL);
-
-	g_free (uri);
-}
-*/
-
-static void
-open_file_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
+open_uri_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
 				const gchar *uri)
 {
-	GList *mime_apps;
-	GnomeVFSMimeApplication *mime_app;
-	gchar *mime_type;
-	gint idx;
+	GAppInfo *app;
 	AnjutaPluginDescription *desc;
-	AnjutaPluginManager *plugin_manager;
-	
-	idx = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menuitem), "index"));
-	desc = (AnjutaPluginDescription*) g_object_get_data (G_OBJECT (menuitem),
-														 "desc");
-	plugin_manager = anjuta_shell_get_plugin_manager (ANJUTA_PLUGIN (plugin)->shell,
-													  NULL);
-	mime_type = anjuta_util_get_uri_mime_type (uri);
-	mime_apps = gnome_vfs_mime_get_all_applications (mime_type);
+	const gchar *mime_type;
 	
 	/* Open with plugin */
+	desc = (AnjutaPluginDescription*) g_object_get_data (G_OBJECT (menuitem),
+														 "desc");
+	mime_type = (const gchar*) g_object_get_data (G_OBJECT (menuitem),
+														 "mime_type");
 	if (desc)
 	{
+		AnjutaPluginManager *plugin_manager;
 		gchar *location = NULL;
+
+		plugin_manager = anjuta_shell_get_plugin_manager (ANJUTA_PLUGIN (plugin)->shell,
+	 														  NULL);
 		
 		anjuta_plugin_description_get_string (desc, "Anjuta Plugin",
 											  "Location", &location);
@@ -717,8 +768,13 @@ open_file_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
 														location);
 			if (loaded_plugin)
 			{
-				ianjuta_file_open (IANJUTA_FILE (loaded_plugin), uri, NULL);
-				set_recent_file (plugin, uri, mime_type);
+				GFile* file = g_file_new_for_uri (uri);
+				GError *error = NULL;
+				
+				ianjuta_file_open (IANJUTA_FILE (loaded_plugin), file, &error);
+				g_object_unref (file);
+				update_recent_file (plugin, uri, mime_type, error == NULL);
+				g_free (error);
 			}
 			else
 			{
@@ -731,20 +787,25 @@ open_file_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
 	}
 	else
 	{
-		GList *uris = NULL;
-		GnomeVFSResult res;
+		/* Open with application */
+		app = (GAppInfo *) g_object_get_data (G_OBJECT (menuitem),
+															 "app");
+		if (app)
+		{
+			GList *uris = NULL;
+			GError *error = NULL;
 		
-		mime_app = g_list_nth_data (mime_apps, idx);
-		uris = g_list_prepend (uris, (gpointer)uri);
-		res = gnome_vfs_mime_application_launch (mime_app, uris);
-		if (res != GNOME_VFS_OK)
-			launch_application_failure (plugin, uri, res);
-		else
-			set_recent_file (plugin, uri, mime_type);
-		g_list_free (uris);
+			uris = g_list_prepend (uris, (gpointer)uri);
+			g_app_info_launch_uris (app, uris, NULL, &error);
+			g_list_free (uris);
+			if (error)
+			{
+				launch_application_failure (plugin, uri, error->message);
+				g_error_free (error);
+			}
+			update_recent_file (plugin, uri, mime_type, error == NULL);
+		}
 	}
-	gnome_vfs_mime_application_list_free (mime_apps);
-	g_free (mime_type);
 }
 
 static void
@@ -758,7 +819,7 @@ static void
 fm_open_with (GtkMenuItem *menuitem, AnjutaFileLoaderPlugin *plugin)
 {
 	if (plugin->fm_current_uri)
-		open_file_with (plugin, menuitem, plugin->fm_current_uri);
+		open_uri_with (plugin, menuitem, plugin->fm_current_uri);
 }
 
 static void
@@ -772,10 +833,18 @@ static void
 pm_open_with (GtkMenuItem *menuitem, AnjutaFileLoaderPlugin *plugin)
 {
 	if (plugin->pm_current_uri)
-		open_file_with (plugin, menuitem, plugin->pm_current_uri);
+		open_uri_with (plugin, menuitem, plugin->pm_current_uri);
 }
 
 static GtkActionEntry actions_file[] = {
+	{ 
+		"ActionFileNew", 
+		GTK_STOCK_NEW,
+		N_("_New"), 
+		"<control>n",
+		N_("New empty file"),
+		G_CALLBACK (on_new_activate)
+	},
 	{
 		"ActionFileOpen",
 		GTK_STOCK_OPEN,
@@ -820,13 +889,12 @@ create_open_with_submenu (AnjutaFileLoaderPlugin *plugin, GtkWidget *parentmenu,
 						  const gchar *uri, GCallback callback,
 						  gpointer callback_data)
 {
-	GList *mime_apps, *node;
-	GnomeVFSMimeApplication *mime_app;
-	GList *plugin_descs, *snode;
+	GList *mime_apps;
+	GList *plugin_descs;
+	GList *node;	
 	GtkWidget *menu, *menuitem;
 	gchar *mime_type;
-	gint idx;
-	gboolean ret;
+	GFile *file;
 	
 	g_return_val_if_fail (GTK_IS_MENU_ITEM (parentmenu), FALSE);
 	
@@ -834,21 +902,20 @@ create_open_with_submenu (AnjutaFileLoaderPlugin *plugin, GtkWidget *parentmenu,
 	gtk_widget_show (menu);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (parentmenu), menu);
 	
-	mime_type = anjuta_util_get_uri_mime_type (uri);
+	file = g_file_new_for_uri (uri);
+	mime_type = anjuta_util_get_file_mime_type (file);
+	g_object_unref (file);
 	if (mime_type == NULL)
 		return FALSE;
 	
-	idx = 0;
-	
 	/* Open with plugins menu items */
-	plugin_descs = get_available_plugins_for_mime (plugin, mime_type);
-	snode = plugin_descs;
-	while (snode)
+	plugin_descs = get_available_plugins_for_mime (ANJUTA_PLUGIN (plugin), mime_type);
+	for (node = plugin_descs; node != NULL; node = g_list_next (node))
 	{
 		gchar *name;
 		AnjutaPluginDescription *desc;
 		
-		desc = (AnjutaPluginDescription *)(snode->data);
+		desc = (AnjutaPluginDescription *)(node->data);
 		name = NULL;
 		anjuta_plugin_description_get_locale_string (desc, "File Loader",
 													 "Title", &name);
@@ -863,63 +930,72 @@ create_open_with_submenu (AnjutaFileLoaderPlugin *plugin, GtkWidget *parentmenu,
 												  "Location", &name);
 		}
 		menuitem = gtk_menu_item_new_with_label (name);
-		g_object_set_data (G_OBJECT (menuitem), "index", GINT_TO_POINTER (idx));
 		g_object_set_data (G_OBJECT (menuitem), "desc", (gpointer)(desc));
+		g_object_set_data (G_OBJECT (menuitem), "mime_type", mime_type);			
 		g_signal_connect (G_OBJECT (menuitem), "activate",
 						  G_CALLBACK (callback), callback_data);
 		gtk_menu_append (menu, menuitem);
 		g_free (name);
-		snode = g_list_next (snode);
-		idx++;
 	}
+	g_list_free (plugin_descs);
 	
 	/* Open with applications */
-	mime_apps = gnome_vfs_mime_get_all_applications (mime_type);
-	if (idx > 0 && mime_apps)
+	mime_apps = g_app_info_get_all_for_type (mime_type);
+	if (plugin_descs && mime_apps)
 	{
 		menuitem = gtk_menu_item_new ();
 		gtk_menu_append (menu, menuitem);
-		idx++;
 	}
-	g_free (mime_type);
-	idx = 0;
-	node = mime_apps;
-	while (node)
+	
+	for (node = mime_apps; node != NULL; node = g_list_next (node))
 	{
-		mime_app = (GnomeVFSMimeApplication *)(node->data);
-		menuitem = gtk_menu_item_new_with_label (mime_app->name);
-		g_object_set_data (G_OBJECT (menuitem), "index", GINT_TO_POINTER (idx));
-		g_signal_connect (G_OBJECT (menuitem), "activate",
-						  G_CALLBACK (callback), callback_data);
-		gtk_menu_append (menu, menuitem);
-		node = g_list_next (node);
-		idx++;
+		GAppInfo *mime_app;
+		
+		mime_app = (GAppInfo *)(node->data);
+		if (g_app_info_should_show (mime_app))
+		{
+			menuitem = gtk_menu_item_new_with_label ( g_app_info_get_name (mime_app));
+			g_object_set_data_full (G_OBJECT (menuitem), "app", mime_app, g_object_unref);			
+			g_object_set_data (G_OBJECT (menuitem), "mime_type", mime_type);			
+			g_signal_connect (G_OBJECT (menuitem), "activate",
+							  G_CALLBACK (callback), callback_data);
+			gtk_menu_append (menu, menuitem);
+		}
+		else
+		{
+			g_object_unref (mime_app);
+		}
 	}
+	g_list_free (mime_apps);
+
 	gtk_widget_show_all (menu);
-	
-	if (mime_apps == NULL && plugin_descs == NULL)
-		ret = FALSE;
+
+	if ((mime_apps != NULL) || (plugin_descs != NULL))
+	{
+		g_object_set_data_full (G_OBJECT (menu), "mime_type", (gpointer)mime_type, g_free);
+		
+		return TRUE;
+	}
 	else
-		ret = TRUE;
-	
-	gnome_vfs_mime_application_list_free (mime_apps);
-	if (plugin_descs)
-		g_list_free (plugin_descs);
-	
-	return ret;
+	{
+		g_free (mime_type);
+		
+		return FALSE;
+	}
 }
 						  
 static void
-value_added_fm_current_uri (AnjutaPlugin *plugin, const char *name,
+value_added_fm_current_file (AnjutaPlugin *plugin, const char *name,
 							const GValue *value, gpointer data)
 {
 	AnjutaUI *ui;
-	const gchar *uri;
+	gchar *uri;
 	AnjutaFileLoaderPlugin *fl_plugin;
 	GtkAction *action;
 	GtkWidget *parentmenu;
+	GFile* file = G_FILE (g_value_get_object (value));
 	
-	uri = g_value_get_string (value);
+	uri = g_file_get_uri (file);
 	g_return_if_fail (name != NULL);
 
 	fl_plugin = ANJUTA_PLUGIN_FILE_LOADER (plugin);
@@ -942,10 +1018,12 @@ value_added_fm_current_uri (AnjutaPlugin *plugin, const char *name,
 	if (!create_open_with_submenu (fl_plugin, parentmenu, uri,
 								   G_CALLBACK (fm_open_with), plugin))
 		g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
+	
+	g_free (uri);
 }
 
 static void
-value_removed_fm_current_uri (AnjutaPlugin *plugin,
+value_removed_fm_current_file (AnjutaPlugin *plugin,
 							  const char *name, gpointer data)
 {
 	AnjutaUI *ui;
@@ -1028,7 +1106,9 @@ value_removed_pm_current_uri (AnjutaPlugin *plugin,
 static void
 dnd_dropped (const gchar *uri, gpointer plugin)
 {
-	ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin), uri, FALSE, NULL);
+	GFile* file = g_file_new_for_uri (uri);
+	ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin), file, FALSE, NULL);
+	g_object_unref (file);
 }
 
 static void
@@ -1036,11 +1116,7 @@ on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
 				 AnjutaSession *session,
 				 AnjutaFileLoaderPlugin *plugin)
 {
-	AnjutaStatus *status;
-	const gchar *uri;
-	gchar *mime_type;
 	GList *files, *node;
-	gint i;
 				
 	/* We want to load the files first before other session loads */
 	if (phase != ANJUTA_SESSION_PHASE_FIRST)
@@ -1049,80 +1125,79 @@ on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
 	files = anjuta_session_get_string_list (session, "File Loader", "Files");
 	if (!files)
 		return;
-	
-	status = anjuta_shell_get_status (shell, NULL);
-	anjuta_status_progress_add_ticks (status, g_list_length(files));
-	
-	/* Open project files first and then regular files */
-	for (i = 0; i < 2; i++)
+		
+	/* Open all files except project files */
+	for (node = g_list_first (files); node != NULL; node = g_list_next (node))
 	{
-		node = files;
-		while (node)
+		gchar *uri = node->data;
+		
+		if (uri)
 		{
-			uri = node->data;
-			if (uri)
+			gchar *fragment;
+			
+			fragment = strchr (uri, '#');
+			if (fragment)
+				*fragment = '\0';
+
+			if (!anjuta_util_is_project_file (uri))
 			{
-				gchar *label, *filename;
-				
-				mime_type = anjuta_util_get_uri_mime_type (uri);
-				
-				filename = g_path_get_basename (uri);
-				if (strchr (filename, '#'))
-					*(strchr (filename, '#')) = '\0';
-				
-				label = g_strconcat ("Loaded: ", filename, NULL);
-				
-				if (i == 0 && mime_type &&
-					strcmp (mime_type, "application/x-anjuta") == 0)
+				GFile* file = g_file_new_for_uri (uri);
+				GObject *loader = ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin),
+											  file, FALSE, NULL);
+				if (fragment)
 				{
-					/* Project files first */
-					/* FIXME: Ignore project files for now */
-					/*
-					ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin),
-											  uri, FALSE, NULL);
-					*/
-					anjuta_status_progress_tick (status, NULL, label);
+					if (IANJUTA_IS_DOCUMENT_MANAGER (loader))
+					{
+						ianjuta_document_manager_goto_file_line (IANJUTA_DOCUMENT_MANAGER (loader), file, atoi(fragment + 1), NULL);
+					}
 				}
-				else if (i != 0 &&
-						 (!mime_type ||
-						  strcmp (mime_type, "application/x-anjuta") != 0))
-				{
-					/* Then rest of the files */
-					ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin),
-											  uri, FALSE, NULL);
-					anjuta_status_progress_tick (status, NULL, label);
-				}
-				g_free (filename);
-				g_free (label);
-				g_free (mime_type);
+				g_object_unref (file);
 			}
-			node = g_list_next (node);
 		}
+		g_free (uri);
 	}
-	if (files)
-	{
-		g_list_foreach (files, (GFunc)g_free, NULL);
-		g_list_free (files);
-	}
+	g_list_free (files);
+}
+
+static void
+setup_recent_chooser_menu (GtkRecentChooser* recent_menu, AnjutaFileLoaderPlugin* plugin)
+{
+	GtkRecentFilter *filter;
+
+	gtk_recent_chooser_set_local_only (GTK_RECENT_CHOOSER (recent_menu), TRUE);
+	gtk_recent_chooser_set_show_icons (GTK_RECENT_CHOOSER (recent_menu), TRUE);
+	gtk_recent_chooser_set_show_tips (GTK_RECENT_CHOOSER (recent_menu), TRUE);
+	gtk_recent_chooser_set_sort_type (GTK_RECENT_CHOOSER (recent_menu), GTK_RECENT_SORT_MRU);
+	gtk_recent_chooser_set_limit (GTK_RECENT_CHOOSER (recent_menu), 20);
+
+	filter = gtk_recent_filter_new ();
+	gtk_recent_filter_add_application (filter, g_get_application_name ());
+	gtk_recent_chooser_set_filter (GTK_RECENT_CHOOSER (recent_menu), filter);
+
+	g_signal_connect (recent_menu, "item-activated",
+					  G_CALLBACK (on_open_recent_file), plugin);	
 }
 
 static gboolean
 activate_plugin (AnjutaPlugin *plugin)
 {
-	EggSubmenuAction *saction;
-	GtkAction *action;
+	GtkAction *action, *saction;
 	AnjutaUI *ui;
 	AnjutaFileLoaderPlugin *loader_plugin;
-	// GtkWidget *recent_menu;
-	// EggRecentViewGtk *recent_view;
 	GtkActionGroup *group;
+	GtkWidget *widget;
+	GtkWidget* recent_menu;
+	GtkWidget* toolbar_menu;
 	
 	loader_plugin = ANJUTA_PLUGIN_FILE_LOADER (plugin);
 	
-	DEBUG_PRINT ("AnjutaFileLoaderPlugin: Activating File Loader plugin...");
+	DEBUG_PRINT ("%s", "AnjutaFileLoaderPlugin: Activating File Loader plugin...");
 	
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	
+	/* Recent manager */
+	loader_plugin->recent_manager = gtk_recent_manager_get_default();
+	
 	/* Add action group */
 	loader_plugin->action_group =
 		anjuta_ui_add_action_group_entries (ui, "ActionGroupLoader",
@@ -1136,13 +1211,9 @@ activate_plugin (AnjutaPlugin *plugin)
 											popup_actions_file,
 											G_N_ELEMENTS (popup_actions_file),
 											GETTEXT_PACKAGE, FALSE, plugin);
-	saction = g_object_new (EGG_TYPE_SUBMENU_ACTION,
-							"name", "ActionFileWizard",
-							"label", _("New"),
-							"tooltip", _("New file, project and project components."),
-							NULL);
-	egg_submenu_action_set_menu_factory (saction,
-										 on_create_submenu, plugin);
+	saction = gtk_recent_action_new ("ActionFileWizard", _("New"),
+							  _("New file, project and project components."), NULL);
+	g_object_set (saction, "stock-id", GTK_STOCK_NEW, NULL);
 	gtk_action_group_add_action (loader_plugin->action_group,
 								 GTK_ACTION (saction));
 	
@@ -1152,17 +1223,11 @@ activate_plugin (AnjutaPlugin *plugin)
 				  "is-important", TRUE, NULL);
 
 	group = gtk_action_group_new ("ActionGroupLoaderRecent");
-	action = g_object_new (EGG_TYPE_RECENT_ACTION,
-						   "name", "ActionFileOpenRecent",
-						   "label", _("Open _Recent"),
-						   "tooltip", _("Open recent file"),
-							NULL);
-	egg_recent_action_add_model (EGG_RECENT_ACTION (action),
-								 loader_plugin->recent_files_model_top);
-	egg_recent_action_add_model (EGG_RECENT_ACTION (action),
-								 loader_plugin->recent_files_model_bottom);
-	g_signal_connect (action, "activate",
-					  G_CALLBACK (on_open_recent_file), plugin);
+	action = gtk_recent_action_new ("ActionFileOpenRecent", _("Open _Recent"),
+												_("Open recent file"), NULL);
+	g_object_set (action, "stock-id", GTK_STOCK_OPEN, NULL);
+	setup_recent_chooser_menu (GTK_RECENT_CHOOSER (action), loader_plugin);
+	
 	gtk_action_group_add_action (group, action);
 	anjuta_ui_add_action_group (ui, "ActionGroupLoaderRecent",
 								N_("Open recent files"), group, FALSE);
@@ -1171,6 +1236,34 @@ activate_plugin (AnjutaPlugin *plugin)
 	/* Add UI */
 	loader_plugin->uiid = anjuta_ui_merge (ui, UI_FILE);
 	
+	/* Adding submenus */
+	recent_menu = anjuta_recent_chooser_menu_new_for_manager (loader_plugin->recent_manager);
+	setup_recent_chooser_menu (GTK_RECENT_CHOOSER (recent_menu), loader_plugin);
+	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER(ui),
+					"/MenuMain/MenuFile/PlaceholderFileMenus/OpenRecent");
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (widget), 
+							   recent_menu);
+	
+	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER(ui),
+					"/MenuMain/MenuFile/PlaceholderFileMenus/Wizard");
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (widget), on_create_submenu(loader_plugin));
+	
+	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER(ui),
+					"/ToolbarMain/PlaceholderFileToolbar/New");
+	gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (widget), on_create_submenu(loader_plugin));
+	g_signal_connect (widget, "clicked", G_CALLBACK (on_new_clicked), loader_plugin);
+	
+	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER(ui),
+					"/ToolbarMain/PlaceholderFileToolbar/Open");
+	toolbar_menu = anjuta_recent_chooser_menu_new_for_manager (loader_plugin->recent_manager);
+	setup_recent_chooser_menu (GTK_RECENT_CHOOSER (toolbar_menu), loader_plugin);
+	gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (widget),
+								   toolbar_menu);
+	gtk_tool_button_set_label (GTK_TOOL_BUTTON (widget), _("Open"));
+	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (widget), _("Open a file"));
+	gtk_menu_tool_button_set_arrow_tooltip_text (GTK_MENU_TOOL_BUTTON (widget), _("Open recent file"));
+	g_signal_connect (widget, "clicked", G_CALLBACK (on_open_clicked), loader_plugin);
+
 	/* Install drag n drop handler */
 	dnd_drop_init (GTK_WIDGET (plugin->shell), dnd_dropped, plugin,
 				   "text/plain", "text/html", "text/source", "application-x/anjuta",
@@ -1178,11 +1271,11 @@ activate_plugin (AnjutaPlugin *plugin)
 	
 	/* Add watches */
 	loader_plugin->fm_watch_id = 
-		anjuta_plugin_add_watch (plugin, "file_manager_current_uri",
-								 value_added_fm_current_uri,
-								 value_removed_fm_current_uri, NULL);
+		anjuta_plugin_add_watch (plugin, IANJUTA_FILE_MANAGER_SELECTED_FILE,
+								 value_added_fm_current_file,
+								 value_removed_fm_current_file, NULL);
 	loader_plugin->pm_watch_id = 
-		anjuta_plugin_add_watch (plugin, "project_manager_current_uri",
+		anjuta_plugin_add_watch (plugin, IANJUTA_PROJECT_MANAGER_CURRENT_URI,
 								 value_added_pm_current_uri,
 								 value_removed_pm_current_uri, NULL);
 	
@@ -1200,7 +1293,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	
 	loader_plugin = ANJUTA_PLUGIN_FILE_LOADER (plugin);
 	
-	DEBUG_PRINT ("AnjutaFileLoaderPlugin: Deactivating File Loader plugin...");
+	DEBUG_PRINT ("%s", "AnjutaFileLoaderPlugin: Deactivating File Loader plugin...");
 	
 	/* Disconnect session */
 	g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->shell),
@@ -1224,18 +1317,14 @@ deactivate_plugin (AnjutaPlugin *plugin)
 static void
 dispose (GObject *obj)
 {
-	AnjutaFileLoaderPlugin *plugin = ANJUTA_PLUGIN_FILE_LOADER (obj);
-	if (plugin->recent_files_model_top)
-	{
-		g_object_unref (plugin->recent_files_model_top);
-		plugin->recent_files_model_top = NULL;
-	}
-	if (plugin->recent_files_model_bottom)
-	{
-		g_object_unref (plugin->recent_files_model_bottom);
-		plugin->recent_files_model_bottom = NULL;
-	}
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
+}
+
+static void
+finalize (GObject *obj)
+{
+	/* Finalization codes here */
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -1245,17 +1334,6 @@ anjuta_file_loader_plugin_instance_init (GObject *obj)
 	
 	plugin->fm_current_uri = NULL;
 	plugin->pm_current_uri = NULL;
-	
-	plugin->recent_files_model_top =
-		egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
-	egg_recent_model_set_limit (plugin->recent_files_model_top, 5);
-	egg_recent_model_set_filter_groups (plugin->recent_files_model_top,
-										"anjuta-projects", NULL);
-	plugin->recent_files_model_bottom =
-		egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
-	egg_recent_model_set_limit (plugin->recent_files_model_bottom, 15);
-	egg_recent_model_set_filter_groups (plugin->recent_files_model_bottom,
-										"anjuta-files", NULL);
 }
 
 static void
@@ -1268,41 +1346,34 @@ anjuta_file_loader_plugin_class_init (GObjectClass *klass)
 	plugin_class->activate = activate_plugin;
 	plugin_class->deactivate = deactivate_plugin;
 	klass->dispose = dispose;
+	klass->finalize = finalize;
 }
 
 static GObject*
-iloader_load (IAnjutaFileLoader *loader, const gchar *uri,
+iloader_load (IAnjutaFileLoader *loader, GFile* file,
 			  gboolean read_only, GError **err)
 {
 	gchar *mime_type;
-	gchar *new_uri;
-	GnomeVFSURI* vfs_uri;
 	AnjutaStatus *status;
 	AnjutaPluginManager *plugin_manager;
 	GList *plugin_descs = NULL;
 	GObject *plugin = NULL;	
+	gchar *uri = g_file_get_uri (file);
 	
 	g_return_val_if_fail (uri != NULL, NULL);
-	vfs_uri = gnome_vfs_uri_new (uri);
-	
-	if (!gnome_vfs_uri_exists (vfs_uri))
-	{
-		launch_application_failure (ANJUTA_PLUGIN_FILE_LOADER (loader),
-									uri, GNOME_VFS_ERROR_NOT_FOUND);
-		gnome_vfs_uri_unref (vfs_uri);
-		return NULL;
-	}
-	
-	new_uri = gnome_vfs_uri_to_string (vfs_uri,
-									   GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
-	mime_type = anjuta_util_get_uri_mime_type (new_uri);
-	gnome_vfs_uri_unref (vfs_uri);
+
+	mime_type = anjuta_util_get_file_mime_type (file);
 	
 	if (mime_type == NULL)
 	{
-		launch_application_failure (ANJUTA_PLUGIN_FILE_LOADER (loader),
-									new_uri, GNOME_VFS_ERROR_NOT_FOUND);
-		g_free (new_uri);
+		update_recent_file (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, NULL, FALSE);	
+		
+		if (err == NULL)
+			launch_application_failure (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, _("File not found"));
+		
+		g_set_error (err, G_IO_ERROR, G_IO_ERROR_NOT_FOUND, _("File not found"));
+		
+		g_free (uri);
 		return NULL;
 	}
 	
@@ -1313,21 +1384,21 @@ iloader_load (IAnjutaFileLoader *loader, const gchar *uri,
 	
 	DEBUG_PRINT ("Opening URI: %s", uri);
 	
-	plugin_descs = anjuta_plugin_manager_query (plugin_manager,
-												"Anjuta Plugin",
-												"Interfaces", "IAnjutaFile",
-												"File Loader",
-												"SupportedMimeTypes",
-												mime_type,
-												NULL);
+	plugin_descs = get_available_plugins_for_mime (ANJUTA_PLUGIN (loader), mime_type);
 	
 	if (g_list_length (plugin_descs) > 1)
 	{
+		gchar* basename = g_path_get_basename (uri);
+		/* %s is name of file that will be opened */ 
+		gchar* message = g_strdup_printf (_("Please select a plugin to open <b>%s</b>."), 
+										 basename);
 		plugin =
 			anjuta_plugin_manager_select_and_activate (plugin_manager,
-													   "Open With",
-								"Please select a plugin to open this file.",
+													   _("Open With"),
+													   message,
 													   plugin_descs);
+		g_free (basename);
+		g_free (message);
 	}
 	else if (g_list_length (plugin_descs) == 1)
 	{
@@ -1344,19 +1415,50 @@ iloader_load (IAnjutaFileLoader *loader, const gchar *uri,
 	}
 	else
 	{
-		launch_in_default_application (ANJUTA_PLUGIN_FILE_LOADER (loader), mime_type, uri);
+		GAppInfo *appinfo;
+		GList *uris = NULL;
+
+		uris = g_list_prepend (uris, (gpointer)uri);
+
+		appinfo = g_app_info_get_default_for_type (mime_type, TRUE);
+		if (appinfo)
+		{
+			GError *error = NULL;
+			if (!g_app_info_launch_uris (appinfo, uris, NULL, &error))
+			{
+				open_with_dialog (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, mime_type);
+			}
+			else
+			{
+				update_recent_file (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, mime_type, error == NULL);
+			}				
+			g_object_unref (G_OBJECT (appinfo));
+		}
+		g_list_free (uris);
 	}
-	if (plugin)
-		ianjuta_file_open (IANJUTA_FILE(plugin), uri, NULL);
 	
-	set_recent_file (ANJUTA_PLUGIN_FILE_LOADER (loader), new_uri, mime_type);
+	if (plugin)
+	{
+		GError *error = NULL;
+		
+		ianjuta_file_open (IANJUTA_FILE(plugin), file, &error);
+		if (error != NULL)
+		{
+			
+			if (err == NULL)
+				launch_application_failure (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, error->message);
+			g_propagate_error (err, error);
+		}
+		update_recent_file (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, mime_type, error == NULL);
+	}
 	
 	if (plugin_descs)
 		g_list_free (plugin_descs);
+	
 	g_free (mime_type);
-	g_free (new_uri);
+	g_free (uri);
 	anjuta_status_busy_pop (status);
-
+		
 	return plugin;
 }
 
